@@ -55,6 +55,9 @@ double ** new_id_matrix(unsigned int n);
 double ** new_zero_matrix(unsigned int n1, unsigned int n2);
 double ** new_matrix(unsigned int m, unsigned int n);
 double ** new_matrix_bones(double *v, unsigned int n1, unsigned int n2);
+int ** new_imatrix_bones(int *v, unsigned int n1, unsigned int n2);
+int ** new_t_imatrix(int** M, unsigned int n1, unsigned int n2);
+int ** new_imatrix(unsigned int n1, unsigned int n2);
 double ** new_t_matrix(double** M, unsigned int n1_old, unsigned int n2_old);
 double ** new_dup_matrix(double** M, unsigned int n1, unsigned int n2);
 double ** new_shift_matrix(double** M, unsigned int n1, unsigned int n2);
@@ -146,6 +149,7 @@ void normv(double *v, unsigned int n, double* norm);
 double sum_fv(double *v, unsigned int n, double(*f)(double));
 double sumv(double *v, unsigned int n);
 double meanv(double *v, unsigned int n);
+int equalv(double *v1, double *v2, int n);
 
 int* new_ivector(unsigned int n);
 int* new_dup_ivector(int *iv, unsigned int n);
